<?php
namespace EMS\PortalFuncionario\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;

use EMS\PortalFuncionario\Geral\Services\HistoricoService;

class HistoricoController extends Crud {

    // protected $dataSourceName = 'employees';

		public function __construct(EntityManager $entityManager, HistoricoService $historicoService) {
			$this->entityManager = $entityManager;
			$this->historicoService = $historicoService;
		}

		public function getHistoricoMarcacoes(DTO\Request\Filter $request, DTO\Response $response) {
			try {
				$params = $request->getParameters();

				$nrorg = $params['NRORG'];
				$nrvinculom = $params['NRVINCULOM'];
				
				$dtinicialapuracao = $params['DTINICIALAPURACAO'];
				$dtfinalapuracao = $params['DTFINALAPURACAO'];

				$historicoMarcacoes = $this->historicoService->getMarcacoes($nrorg, $nrvinculom, 
				$dtinicialapuracao, $dtfinalapuracao);

				$response->addDataSet(new DataSet('historico', $historicoMarcacoes));
			} catch (\Exception $e) {
				$response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
			}
		}

		public function getQuantidadeMarcacoesDia(DTO\Request\Filter $request, DTO\Response $response) {
			try {
				$params = $request->getParameters();

				$nrorg = $params['NRORG'];
				$nrvinculom = $params['NRVINCULOM'];
				$dtapuracao = $params['DTAPURACAO'];

				$marcacoesDia = $this->historicoService->getQuantidadeMarcacoes($nrorg, $nrvinculom, $dtapuracao);

				$response->addDataSet(new DataSet('quantidadeMarcacoesHistorico', array('quantidadeMarcacoes' => $marcacoesDia)));
			} catch (\Exception $e) {
				$response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
			}
		}

		public function getUltimaMarcacao(DTO\Request\Filter $request, DTO\Response $response) {
			try {
				$params = $request->getParameters();

				$nrorg = $params['NRORG'];
				$nrvinculom = $params['NRVINCULOM'];

				$ultimaMarcacao = $this->historicoService->getUltimaMarcacao($nrorg, $nrvinculom);

				$response->addDataSet(new DataSet('ultimaMarcacaoHistorico', $ultimaMarcacao));
			} catch (\Exception $e) {
				$response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
			}
		}

}
