<?php
namespace EMS\PortalFuncionario\Geral\Repository;

use Zeedhi\Framework\Controller\Crud;
use Doctrine\ORM\EntityManager;

use EMS\PortalFuncionario\Util\{DateUtil, NovoCodigo};

class MarcacaoRepository extends Crud {

    const TIPO_VINC_CLT = 1;
    const VINC_EMPREG_APRENDIZ = 9;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    /*
    * Função pra poder montar o novo regitro de marcação que será inserido
    */
    public function getHorarioBase(string $nrvinculom, string $nrorg) : array {
        try {
            $dadosHorarioVinculo = $this->getDadosHorarioVinculo($nrvinculom, $nrorg);
            $nrhordiam = $dadosHorarioVinculo['NRHORDIAM'];

            $toleranciaAnterior  = DateUtil::getHoraFloatDeString(
                $this->entityManager->getRepository('Teknisa\Model\Gestaopessoas\Entities\GpeParamorg')->findOneBy(array('nrorg' => $nrorg))->getHrtempotoleranciaant());
            $toleranciaPosterior = DateUtil::getHoraFloatDeString(
                $this->entityManager->getRepository('Teknisa\Model\Gestaopessoas\Entities\GpeParamorg')->findOneBy(array('nrorg' => $nrorg))->getHrtempotoleranciapos());


            $sql = <<<SQL

                SELECT
                    --TO_CHAR(TO_DATE(HRENTRADA, 'HH24MI') - 0.003472, 'HH24:MI') AS HRENTRADA_NOVA,
                    SUBSTR(SUBSTR(HRENTRADA, 1, LENGTH(TRIM(HRENTRADA)) -2) || ':' || SUBSTR(HRENTRADA, -2), 1, 10) AS HRENTRADA,
                    SUBSTR(SUBSTR(HRPROJETARINTERVALO, 1, LENGTH(TRIM(HRPROJETARINTERVALO)) -2) || ':' || SUBSTR(HRPROJETARINTERVALO, -2), 1, 10) AS HRPROJETARINTERVALO,
                    (SELECT SUBSTR(SUBSTR(HRSAIDA, 1, LENGTH(TRIM(HRSAIDA)) -2) || ':' || SUBSTR(HRSAIDA, -2), 1, 10) AS HRSAIDA
                        FROM GPE_INTERVALOSHORARIO IH
                    WHERE IH.NRORG     = :P_NRORG
                        AND IH.NRHORDIAM = :P_NRHORDIAM
                        AND IH.NRSEQINTERVHORARIO = (SELECT MAX(NRSEQINTERVHORARIO)
                                                            FROM GPE_INTERVALOSHORARIO
                                                        WHERE NRORG     = IH.NRORG
                                                            AND NRHORDIAM = IH.NRHORDIAM)) HRSAIDA
                FROM GPE_INTERVALOSHORARIO IH
                    
                WHERE IH.NRORG     = :P_NRORG
                    AND IH.NRHORDIAM = :P_NRHORDIAM
                    AND IH.NRSEQINTERVHORARIO = (SELECT MIN(NRSEQINTERVHORARIO)
                                                    FROM GPE_INTERVALOSHORARIO
                                                    WHERE NRORG     = IH.NRORG
                                                    AND NRHORDIAM = IH.NRHORDIAM)
            
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"     => $nrorg,
                    "P_NRHORDIAM" => $nrhordiam
                )
            );

            $horariobase = $query->fetchAll();        

            if (count($horariobase) > 0) {
                $horariobase[0]["HRENTRADA_NOVA"] = DateUtil::getHoraStringDeFloat(DateUtil::getHoraFloatDeString($horariobase[0]["HRENTRADA"]) - $toleranciaAnterior, true);
                $horariobase[0]["HRSAIDA_NOVA"]   = DateUtil::getHoraStringDeFloat(DateUtil::getHoraFloatDeString($horariobase[0]["HRSAIDA"]) + $toleranciaPosterior, true);
                
                return $horariobase[0];
            } else {
                throw new \Exception("Horário base não encontrado.", 1);          
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function insereMarcacoesPontoManual(
        string $nrvinculom,
        string $dthrmarcacaoponto,
        string $dthrajustada,
        string $idenquadrahorario,
        string $idmarcdesprezada,
        string $iddiaocorrencia,
        string $dsjustificativa,
        string $nrorginclusao,
        int    $nrcoletor,
        string $idreferintervalo,
               $cdcnpjestrut,
               $nmrazsocestrut,
               $cdcaepfestrut,
               $cdceiestrut,
        string $cdticketaut,
        string $dslatitude,
        string $dslongitude,
               $dspathfoto,
        string $idativo,
        string $cdoperinclusao,
        string $nrorg
    ) : string {

        $nrmarcacaoponto = NovoCodigo::geraCodigo($nrorg, 'GPE_MARCACAOPONTO', 1, 22, false);

        $dtinclusao = $dthrmarcacaoponto;

        $sql = <<<SQL
            INSERT INTO GPE_MARCACAOPONTO (
                NRMARCACAOPONTO,
                NRVINCULOM,
                DTHRMARCACAOPONTO,
                DTHRAJUSTADA,
                IDENQUADRAHORARIO,
                IDMARCDESPREZADA,
                IDDIAOCORRENCIA,
                DSJUSTIFICATIVA,
                NRORGINCLUSAO,
                NRCOLETOR,
                IDREFERINTERVALO,
                CDCNPJESTRUT,
                NMRAZSOCESTRUT,
                CDCAEPFESTRUT,
                CDCEIESTRUT,
                CDTICKETAUT,
                DSLATITUDE,
                DSLONGITUDE,
                DSPATHFOTO,
                IDATIVO,
                DTINCLUSAO,
                CDOPERINCLUSAO,
                NRORG
            ) VALUES (
                :P_NRMARCACAOPONTO,
                :P_NRVINCULOM,
                TO_DATE(:P_DTHRMARCACAOPONTO, 'DD/MM/YYYY HH24:MI:SS'),
                TO_DATE(:P_DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI:SS'),
                :P_IDENQUADRAHORARIO,
                :P_IDMARCDESPREZADA,
                :P_IDDIAOCORRENCIA,
                :P_DSJUSTIFICATIVA,
                :P_NRORGINCLUSAO,
                :P_NRCOLETOR,
                :P_IDREFERINTERVALO,
                :P_CDCNPJESTRUT,
                :P_NMRAZSOCESTRUT,
                :P_CDCAEPFESTRUT,
                :P_CDCEIESTRUT,
                :P_CDTICKETAUT,
                :P_DSLATITUDE,
                :P_DSLONGITUDE,
                :P_DSPATHFOTO,
                :P_IDATIVO,
                SYSDATE,
                :P_CDOPERINCLUSAO,
                :P_NRORG
            )
SQL;

        $query = $this->entityManager->getConnection()->prepare($sql);

        $result = $query->execute(
            array(
                "P_NRMARCACAOPONTO"     => $nrmarcacaoponto,
                "P_NRVINCULOM"          => $nrvinculom,
                "P_DTHRMARCACAOPONTO"   => $dthrmarcacaoponto,
                "P_DTHRAJUSTADA"        => $dthrajustada,
                "P_IDENQUADRAHORARIO"   => $idenquadrahorario,
                "P_IDMARCDESPREZADA"    => $idmarcdesprezada,
                "P_IDDIAOCORRENCIA"     => $iddiaocorrencia,
                "P_DSJUSTIFICATIVA"     => $dsjustificativa,
                "P_NRORGINCLUSAO"       => $nrorginclusao,
                "P_NRCOLETOR"           => $nrcoletor,
                "P_IDREFERINTERVALO"    => $idreferintervalo,
                "P_CDCNPJESTRUT"        => $cdcnpjestrut,
                "P_NMRAZSOCESTRUT"      => $nmrazsocestrut,
                "P_CDCAEPFESTRUT"       => $cdcaepfestrut,
                "P_CDCEIESTRUT"         => $cdceiestrut,
                "P_CDTICKETAUT"         => $cdticketaut,
                "P_DSLATITUDE"          => $dslatitude,
                "P_DSLONGITUDE"         => $dslongitude,
                "P_DSPATHFOTO"          => $dspathfoto,
                "P_IDATIVO"             => $idativo,
                "P_CDOPERINCLUSAO"      => $cdoperinclusao,
                "P_NRORG"               => $nrorg
            )
        );

        if ($result) {
            return $nrmarcacaoponto;
        } else {
            return false;
        }
    }

    public function getDadosHorarioVinculo(string $nrvinculom, string $nrorg) : array {

        try {
            $dtapuracaoponto = new \DateTime();
            $dtapuracaoponto = $dtapuracaoponto->format('d/m/Y');

            $gpeVinculom = $this->entityManager->getRepository('Teknisa\Model\Gestaopessoas\Entities\GpeVinculom')->findOneBy(array('nrvinculom' => $nrvinculom, 'nrorg' => $nrorg));
            $nrtipovinculom  = $gpeVinculom->getNrtipovinculom();
            $nrvinculoempreg = $gpeVinculom->getNrvinculoempreg();

            $whereHrprojetarintervalo = "";
            if ($nrtipovinculom == self::TIPO_VINC_CLT && $nrvinculoempreg != self::VINC_EMPREG_APRENDIZ) {
                $whereHrprojetarintervalo = " AND (IH.HRPROJETARINTERVALO IS NOT NULL OR IDTIPOHORARIO = 'F')";
            }

            $sql = <<<SQL

                SELECT HT.NRHORDIAM, HD.IDTIPOHORARIO, IH.NRINTERVALOHORARIO, IV.HRPROJETARINTERVALO, HH.NMHORDIAH, AE.NRESCALATRABM, EH.QTHRESCALATRABH AS QTJORNADADIARIA
                    FROM (SELECT NRORG, MAX(NRTURNO) NRTURNO, NRESCALATRABM
                        FROM GPE_ALTEESCALA
                        WHERE NRORG = :P_NRORG
                            AND NRVINCULOM = :P_NRVINCULOM
                            --AND TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') BETWEEN DTINIESCALA AND NVL(DTFIMESCALA, TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY'))
                                    AND DTINIESCALA = (SELECT MAX(DTINIESCALA)
                                                FROM GPE_ALTEESCALA ALT
                                                WHERE ALT.NRORG        = GPE_ALTEESCALA.NRORG
                                                AND ALT.NRVINCULOM   = GPE_ALTEESCALA.NRVINCULOM
                                                AND ALT.DTINIESCALA <= TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY'))
                        GROUP BY NRORG, NRESCALATRABM) AE,
                        (SELECT NRORG, NRTURNO, TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY')-DTDATABASE AS NRDIASENTRE
                        FROM GPE_TURNO
                        WHERE NRORG = :P_NRORG) TU,
                        (SELECT NRORG, NRTURNO, NRHORDIAM,
                            ROW_NUMBER() OVER (PARTITION BY NRORG, NRTURNO ORDER BY NRSEQHORARIO) AS NRSEQHR
                        FROM GPE_HORARIOTURNO
                        WHERE NRORG = :P_NRORG
                        ORDER BY NRORG, NRTURNO, NRSEQHORARIO) HT,
                        (SELECT NRORG, NRTURNO, COUNT(*) AS QTHORARIOS
                        FROM GPE_HORARIOTURNO
                        WHERE NRORG = :P_NRORG
                        GROUP BY NRORG, NRTURNO) QH,
                        (SELECT NRORG, NRHORDIAM, MAX(DTMESCOMPETENC) AS DTMESCOMPETENC, NMHORDIAH
                        FROM GPE_HORDIAH
                        WHERE NRORG = :P_NRORG
                            AND DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY')
                        group by NRORG, NRHORDIAM, NMHORDIAH) HH,
                        (SELECT NRORG, NRHORDIAM, MIN(NRSEQINTERVHORARIO) NRSEQINTERVHORARIO, HRPROJETARINTERVALO
                        FROM GPE_INTERVALOSHORARIO
                        where NRORG = :P_NRORG
                        GROUP BY NRORG, NRHORDIAM, HRPROJETARINTERVALO) IV, GPE_INTERVALOSHORARIO IH, GPE_HORDIAH HD, GPE_ESCALATRABH EH
                    WHERE AE.NRORG = TU.NRORG
                    AND AE.NRTURNO = TU.NRTURNO
                    AND AE.NRORG = HT.NRORG
                    AND AE.NRTURNO = HT.NRTURNO
                    AND HT.NRORG = HH.NRORG
                    AND HT.NRHORDIAM = HH.NRHORDIAM
                    AND HD.NRORG = HH.NRORG
                    AND HD.NRHORDIAM = HH.NRHORDIAM
                    AND HD.DTMESCOMPETENC = HH.DTMESCOMPETENC
                    AND IV.NRORG(+) = HH.NRORG
                    AND IV.NRHORDIAM(+) = HH.NRHORDIAM
                    AND IH.NRORG(+) = IV.NRORG
                    AND IH.NRHORDIAM(+) = IV.NRHORDIAM
                    AND IH.NRSEQINTERVHORARIO(+) = IV.NRSEQINTERVHORARIO
                    AND AE.NRORG = QH.NRORG
                    AND AE.NRTURNO = QH.NRTURNO
                    and HT.NRSEQHR-1 = MOD(TU.NRDIASENTRE, QH.QTHORARIOS)
                    $whereHrprojetarintervalo
                    AND EH.NRESCALATRABM = AE.NRESCALATRABM
                    AND EH.NRORG = AE.NRORG
                    AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                                FROM GPE_ESCALATRABH
                                                WHERE NRORG = EH.NRORG
                                                and NRESCALATRABM = AE.NRESCALATRABM
                                                AND DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY'))
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"            => $nrorg,
                    "P_NRVINCULOM"       => $nrvinculom,
                    "P_DTMESCOMPETENCIA" => DateUtil::getPrimeiroDiaDoMes(new \DateTime())->format(DateUtil::FORMATO_BRASILEIRO),
                    "P_DTAPURACAOPONTO"  => $dtapuracaoponto
                )
            );

            $dadosHorario = $query->fetchAll();

            if (isset($dadosHorario[0])) {
                return $dadosHorario[0];
            } else {
                return false;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getSqlTime() {
        $sql = <<<SQL
            SELECT TO_CHAR(SYSDATE, 'DD/MM/YYYY') DATE_DMY,
                   TO_CHAR(SYSDATE, 'HH24:MI:SS') TIME_H24
            FROM DUAL
SQL;

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute();

        return $std->fetch();
    }
}
