<?php

namespace EMS\PortalFuncionario\Util;

use Zeedhi\Framework\Remote\FileUploader\Server;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

use GuzzleHttp\Client;

class RequestUtils {

    public function __construct(string $projectPath, string $loginBackendPath) {
        $this->projectPath          = $projectPath;
        $this->loginBackendPath     = $loginBackendPath;
    }

    const DEFAULT_HEADERS = array(
        'Content-Type: application/json'
    );

    public static function request(
        string  $uri,
        array   $data,
        string  $method,
                $headers = self::DEFAULT_HEADERS
    ) {
        $dataHeader = array('Content-Length: ' . strlen(json_encode($data)));
        $completeHeaders = array_merge($headers, $dataHeader);

        $http = new Client([
            'base_uri' => $uri
        ]);

        $loginApiresponse = $http->request($method, $uri, [
            'json' => $data,
            'headers' => $completeHeaders
        ]);

        $content = $loginApiresponse->getBody()->getContents();
        $body = json_decode($content, true);

        return $body;
    }

    // Inutilizado. Precisam ser adicionadas tratativas para obter o caminho de teste e produção
    public function makeLoginRequest(
        $email,
        $password,
        $productId,
        $requesterUrl,
        $attempts
    ) {
        $loginRoute = $this->getLoginRoute();
        $loginMethod = 'POST';

        $data = array(
            'requestType' => 'FilterData',
            'filter' => array(
                array(
                    'name'      => 'EMAIL',
                    'operator'  => '=',
                    'value'     => $email
                ),
                array(
                    'name'      => 'PASSWORD',
                    'operator'  => '=',
                    'value'     => $password
                ),
                array(
                    'name'      => 'PRODUCT_ID',
                    'operator'  => '=',
                    'value'     => $productId
                ),
                array(
                    'name'      => 'REQUESTER_URL',
                    'operator'  => '=',
                    'value'     => $requesterUrl
                ),
                array(
                    'name'      => 'ATTEMPTS',
                    'operator'  => '=',
                    'value'     =>  $attempts
                ),
            ),
        );

        $response = self::request(
            $loginRoute,
            $data,
            $loginMethod
        );

        return $response;
    }

    public function getLoginRoute() {
        $loginRoute = $this->projectPath . $this->loginBackendPath . "/index.php/login";
        return $loginRoute;
    }
}
